;; Copyright (c) Iwata <iwata@quasiquote.org>
;;
;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal in the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:
;;
;; The above copyright notice and this permission notice shall be included
;; in all copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;; IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
;; CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
;; TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
;; SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(declare
 (unit twitter-config)
 (export
  get-config-path
  read-config
  store-config
  config-filter?
  config-interval?
  config-name?
  config-proxy?
  ))

(use srfi-1)
(use srfi-9)
(use utils)
(use regex)

#>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>

char *
get_home_path(void)
{
  struct passwd *pw;
  char *ret;

  if ((pw = getpwuid(getuid())) == NULL)
	  return NULL;
  ret = strdup(pw->pw_dir);
  endpwent();
  return ret;
}
<#

(define (get-config-path)
  (string-append ((foreign-safe-lambda c-string* "get_home_path"))
                 "/.twitter-notifier"))

(define (get-config-file-path)
  (make-pathname (get-config-path)
                 "config"))

(define (read-config)
  (call-with-input-file
      (get-config-file-path)
    (lambda (port)
      (let loop ((ret (read port))
                 (buf '()))
        (if (eof-object? ret)
            (reverse buf)
            (loop (read port) (cons ret buf)))))))

(define-record-type config-data
  (make-config-data filter interval name proxy) config-data?
  (filter filter? filter!)
  (interval interval? interval!)
  (name name? name!)
  (proxy proxy? proxy!)
  )

(define (config-filter? config)
  (filter? config))
(define (config-interval? config)
  (interval? config))
(define (config-name? config)
  (name? config))
(define (config-proxy? config)
  (proxy? config))


(define (store-config config-list)
  (let ((config (make-config-data '()
                                  60
                                  (foreign-safe-lambda c-string "getlogin")
                                  '()
                                  )))
    (for-each (lambda (entry)
                (define (entry? name) (eq? (car entry) name))
                (define (item?) (cadr entry))
                (cond ((entry? 'interval)
                       (if (number? (item?))
                           (interval! config (item?))
                           (error "invalid entry. interval must be number.")))
                      ((entry? 'filter)
                       (for-each (lambda (fs)
                                   (if (pair? fs)
                                       (filter! config (cons (cons (car fs) ;; match name or text
                                                                   (cons (regexp (cadr fs)) ;; regexp, compile now
                                                                         (cddr fs))) ;; timeout
                                                             (filter? config)))
                                       (error "invalid entry. filter must be alist.")))
                                 (item?)))
                      ((entry? 'name)
                       (if (string? (item?))
                           (name! config (item?))
                           (error "invalid entry. name must be string.")))
                      ((entry? 'proxy)
                       (if (and (pair? (item?))
                                (string? (car (item?)))
                                (number? (cdr (item?))))
                           (proxy! config (item?))
                           (error "invalid entry. proxy must be pair.")))
                      (else
                       (error (format "unknown entry \"~a\"." entry)))
                ))
              config-list)
    config))
